#define IPv4 257
#define IPv6 258
#define HOSTADDR 259
#define WORD 260
#define STRING 261
#define INTEGER 262
#define SYSLOG_BANNER 263
#define TIMESTAMP_SYSLOG 264
#define TIMESTAMP_ISO8601 265
#define TIMESTAMP_TAI64 266
#define AT_TIMESTAMP_TAI64 267
#define RFC_5234_BANNER 268
#define METALOG_BANNER 269
#define SOCKLOG_BANNER 270
#define BUSYBOX_SYSLOG_BANNER 271
#define REPETITIONS 272
#define SSH_INVALUSERPREF 273
#define SSH_NOTALLOWEDPREF 274
#define SSH_NOTALLOWEDSUFF 275
#define SSH_LOGINERR_PREF 276
#define SSH_LOGINERR_PAM 277
#define SSH_VIA 278
#define SSH_MAXAUTH 279
#define SSH_ADDR_SUFF 280
#define SSH_NOIDENTIFSTR 281
#define SSH_BADPROTOCOLIDENTIF 282
#define SSH_BADPROTOCOLIDENTIF_SUFF 283
#define SSH_INVALIDFORMAT_PREF 284
#define SSH_INVALIDFORMAT_SUFF 285
#define SSH_BADKEX_PREF 286
#define SSH_BADKEX_SUFF 287
#define SSH_DISCONNECT_PREF 288
#define SSH_CONNECTION_CLOSED 289
#define SSH_PREAUTH_SUFF 290
#define SSHGUARD_ATTACK_PREF 291
#define SSHGUARD_ATTACK_SUFF 292
#define SSHGUARD_BLOCK_PREF 293
#define SSHGUARD_BLOCK_SUFF 294
#define BIND_PREF 295
#define BIND_QUERY_DENIED 296
#define DOVECOT_IMAP_LOGINERR_PREF 297
#define DOVECOT_IMAP_LOGINERR_SUFF 298
#define UWIMAP_LOGINERR 299
#define CYRUSIMAP_SASL_LOGINERR_PREF 300
#define CYRUSIMAP_SASL_LOGINERR_SUFF 301
#define CUCIPOP_AUTHFAIL 302
#define EXIM_ESMTP_AUTHFAIL_PREF 303
#define EXIM_ESMTP_AUTHFAIL_SUFF 304
#define EXIM_ESMTP_LOGINFAIL_PREF 305
#define EXIM_ESMTP_LOGINFAIL_SUFF 306
#define SENDMAIL_RELAYDENIED_PREF 307
#define SENDMAIL_RELAYDENIED_SUFF 308
#define SENDMAIL_AUTHFAILURE_PREF 309
#define SENDMAIL_AUTHFAILURE_SUFF 310
#define POSTFIX_NO_AUTH_PREF 311
#define POSTFIX_SASL_LOGINERR_PREF 312
#define POSTFIX_SASL_LOGINERR_SUFF 313
#define POSTFIX_NONSMTP 314
#define POSTFIX_NONSMTP_SUFF 315
#define POSTFIX_GREYLIST 316
#define POSTFIX_GREYLIST_SUFF 317
#define POSTSCREEN_PREF 318
#define POSTSCREEN_SUFF 319
#define FREEBSDFTPD_LOGINERR_PREF 320
#define FREEBSDFTPD_LOGINERR_SUFF 321
#define PROFTPD_LOGINERR_PREF 322
#define PROFTPD_LOGINERR_SUFF 323
#define PUREFTPD_LOGINERR_PREF 324
#define PUREFTPD_LOGINERR_SUFF 325
#define VSFTPD_LOGINERR_PREF 326
#define VSFTPD_LOGINERR_SUFF 327
#define COCKPIT_AUTHFAIL_PREF 328
#define COCKPIT_AUTHFAIL_SUFF 329
#define CLF_TIMESTAMP 330
#define CLF_SUFFIX 331
#define CLF_WEB_PROBE 332
#define CLF_CMS_LOGIN 333
#define OPENSMTPD_FAILED_CMD_PREF 334
#define OPENSMTPD_AUTHFAIL_SUFF 335
#define OPENSMTPD_UNSUPPORTED_CMD_SUFF 336
#define COURIER_AUTHFAIL_PREF 337
#define OPENVPN_TLS_ERR_SUFF 338
#define GITEA_ERR_PREF 339
#define GITEA_ERR_SUFF 340
#define OPENVPN_PS_TERM_PREF 341
#define OPENVPN_PS_TERM_SUFF 342
#define MSSQL_AUTHFAIL_PREF 343
#define PROXMOXVE_AUTHFAIL_PREF 344
#define PROXMOXVE_AUTHFAIL_SUFF 345
#ifdef YYSTYPE
#undef  YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#endif
#ifndef YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
typedef union {
    char *str;
    int num;
} YYSTYPE;
#endif /* !YYSTYPE_IS_DECLARED */
extern YYSTYPE yylval;
